precision highp float;

#define OCTAVES 6

highp float random(vec2 co)
{
    highp float a = 12.9898;
    highp float b = 78.233;
    highp float c = 43758.5453;
    highp float dt= dot(co.xy ,vec2(a,b));
    highp float sn= mod(dt,3.14);
    return fract(sin(sn) * c);
}

float noise (in vec2 st) {
    vec2 i = floor(st);
    vec2 f = fract(st);

    // Four corners in 2D of a tile
    float a = random(i);
    float b = random(i + vec2(1.0, 0.0));
    float c = random(i + vec2(0.0, 1.0));
    float d = random(i + vec2(1.0, 1.0));

    // Smooth Interpolation

    // Cubic Hermine Curve.  Same as SmoothStep()
    vec2 u = f*f*(3.0-2.0*f);
    // u = smoothstep(0.,1.,f);

    // Mix 4 coorners percentages
    //return a;
    return mix(a, b, u.x) +
        (c - a)* u.y * (1.0 - u.x) +
        (d - b) * u.x * u.y;
}

float fbm (in vec2 st) {
    // Initial values
    float value = 0.0;
    float amplitude = .5;
    float frequency = 0.;
    //
    // Loop of octaves
    for (int i = 0; i < OCTAVES; i++) {
        value += amplitude * noise(st);
        st *= 2.;
        amplitude *= .5;
    }
    return value;
}
const int count = 50;

vec3 blur(vec2 st){
    vec4 color = vec4(0, 0, 0, 0);
    for(int i = -count; i <= count; i++) {
        float v = (random(st + vec2(i, i))  + 0.2) / (float(count) * 2.0 + 1.0);
        color = color + vec4(vec3(v), 1.0); 
    }
    return color.xyz;
}

void main() {
    vec2 st = gl_FragCoord.xy/600.0;

    vec3 color1 = vec3(0.0);
    color1 += fbm(st*1.0);

    vec3 color2 = blur(gl_FragCoord.xy);

    float v =  (1.0 - gl_FragCoord.x*1.0/600.0 + gl_FragCoord.y*1.0/600.0 ) / 2.0;
    vec3 color3 = vec3(v);

    vec3 color = mix(color3, mix(color1, color2, 0.9), 0.8);

    gl_FragColor = vec4(color.x, color.y, color.z, 1.0);
}

